package cn.reghao.oss.console.app.db.repository;

import cn.reghao.oss.console.app.model.po.UploadChannel;
import org.springframework.data.jpa.repository.JpaRepository;

import java.util.List;

/**
 * @author reghao
 * @date 2024-02-23 15:05:30
 */
public interface UploadChannelRepository extends JpaRepository<UploadChannel, Integer> {
    int countByCreateBy(int createBy);
    List<UploadChannel> findByCreateBy(int createBy);
    List<UploadChannel> findByCreateByAndNodeId(int createBy, int nodeId);
    UploadChannel findByCreateByAndChannelId(int createBy, int channelId);
    UploadChannel findByCreateByAndPrefix(int createBy, String prefix);
    UploadChannel findById(int id);
}
